
/**
 * Classe que representa a informao sobre uma dada
 * conta bancria.
 * @author F. Mrio Martins
 * @version 1.0/2005
 */

import java.util.GregorianCalendar;
import java.io.*;
public abstract class Conta implements Serializable {

    // Variveis de Instncia
    private String numConta;
    private String titular;
    private GregorianCalendar dataInicio;
    private GregorianCalendar dataUltMov;
    // Construtores
    public Conta(String num, String tit,
                     GregorianCalendar dataInic,
                     GregorianCalendar dataUMov) {
       numConta = num; titular = tit; 
       dataInicio = (GregorianCalendar) dataInic.clone();
       dataUltMov = (GregorianCalendar) dataUMov.clone(); 
    }
    
    public Conta(Conta cb) {
        numConta = cb.getNumConta(); titular = cb.getTitular();
        dataInicio = (GregorianCalendar) cb.getInicio().clone();
        dataUltMov = (GregorianCalendar) cb.getUltMov().clone();
    }
    
    // Mtodos de Instncia
    public String getNumConta() { return numConta; }

    public String getTitular() { return titular; }
    public GregorianCalendar getInicio() { return dataInicio; }
    public GregorianCalendar getUltMov() { return dataUltMov; }
    public void mudaUltMov(GregorianCalendar dataMov) { 
                        dataUltMov = dataMov; }
                        
    /**
     * Mtodo que d a valorizao actual de cada conta.
     * 
     *  declarado nesta classe como abstracto, porque a sua implementao
     * depende do tipo de conta.
     */                    
    public abstract double getValor();
                        
    public String toString() {
        StringBuilder s = new StringBuilder();
        s.append("------------------------------------------------------\n");
        s.append("Conta N: " + numConta + "\t\t\t");
        s.append("Titular: " + titular + "\n");
        int ano = dataInicio.get(GregorianCalendar.YEAR);
        int mes = dataInicio.get(GregorianCalendar.MONTH);
        int dia = dataInicio.get(GregorianCalendar.DAY_OF_MONTH);
        s.append("Inicio: " + dia  + "/" + mes +  "/" + ano + "\t\t");
        ano = dataUltMov.get(GregorianCalendar.YEAR);
        mes = dataUltMov.get(GregorianCalendar.MONTH);
        dia = dataUltMov.get(GregorianCalendar.DAY_OF_MONTH);
        s.append("Ult. Mov: " + dia + "/" + mes + "/" + ano + "\n");
        return s.toString();
    }
    
    
    public boolean equals(Object o) {
      if (this == o) return true;
      if ((o == null) || (this.getClass() != o.getClass()))
       return false;
  
      Conta umaConta = (Conta)o;
      
      return (this.numConta.equals(umaConta.getNumConta()) && 
              this.titular.equals(umaConta.getTitular()) &&
              this.dataInicio.equals(umaConta.getInicio()) &&
              this.dataUltMov.equals(umaConta.getUltMov()));
    }  
       
  public abstract Conta clone();
       
}